libBrokenLocale {
  GLIBC_2.0
}
libc {
  GLIBC_2.0
  GLIBC_2.1
  GLIBC_2.1.1
  GLIBC_2.1.2
  GLIBC_2.1.3
  GLIBC_2.1.4
  GLIBC_2.2
  GLIBC_2.2.1
  GLIBC_2.2.2
  GLIBC_2.2.3
  GLIBC_2.2.4
  GLIBC_2.2.5
  GLIBC_2.2.6
  GLIBC_2.3
  GLIBC_2.3.1
  GLIBC_2.3.2
  GLIBC_2.3.3
  GLIBC_2.3.4
  GLIBC_2.4
  GLIBC_2.5
  GLIBC_2.6
  GLIBC_2.7
  GLIBC_2.8
  GLIBC_2.9
  GLIBC_2.10
  GLIBC_2.11
%ifdef USE_IN_LIBIO
  HURD_CTHREADS_0.3
%endif
%ifdef EXPORT_UNWIND_FIND_FDE
  GCC_3.0
%endif
  GLIBC_PRIVATE
}
libcrypt {
  GLIBC_2.0
}
libdl {
  GLIBC_2.0
  GLIBC_2.1
  GLIBC_2.3.3
  GLIBC_2.3.4
}
libm {
  GLIBC_2.0
  GLIBC_2.1
  GLIBC_2.2
  GLIBC_2.2.3
  GLIBC_2.3.4
  GLIBC_2.4
}
libnsl {
  GLIBC_2.0
  GLIBC_2.1
  GLIBC_2.2
}
libnss_compat {
  GLIBC_PRIVATE
}
libnss_dns {
  GLIBC_PRIVATE
}
libnss_files {
  GLIBC_PRIVATE
}
libnss_hesiod {
  GLIBC_PRIVATE
}
libnss_nis {
  GLIBC_PRIVATE
}
libnss_nisplus {
  GLIBC_PRIVATE
}
libpthread {
  GLIBC_2.0
  GLIBC_2.1
  GLIBC_2.1.1
  GLIBC_2.1.2
  GLIBC_2.2
  GLIBC_2.2.3
  GLIBC_2.2.6
  GLIBC_2.3.2
  GLIBC_2.3.3
  GLIBC_2.3.4
  GLIBC_2.4
  GLIBC_2.6
  GLIBC_2.11
  GLIBC_PRIVATE
}
libresolv {
  GLIBC_2.0
  GLIBC_2.2
  GLIBC_2.3.2
  GLIBC_2.9
  GLIBC_PRIVATE
}
librt {
  GLIBC_2.1
  GLIBC_2.2
  GLIBC_2.3
  GLIBC_2.3.3
  GLIBC_2.3.4
  GLIBC_2.4
  GLIBC_2.7
}
libutil {
  GLIBC_2.0
  GLIBC_2.9
}
ld {
  GLIBC_2.0
  GLIBC_2.1
  GLIBC_2.3
  GLIBC_2.4
  GLIBC_PRIVATE
}
libthread_db {
  GLIBC_2.1.3
  GLIBC_2.2.3
  GLIBC_2.3
  GLIBC_2.3.3
}
libanl {
  GLIBC_2.2.3
}
libcidn {
  GLIBC_PRIVATE
}
